/**
 * @(#)tsid2.h 2002/02/25
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID2.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _TSID2_H
#define _TSID2_H

#include "sidt2.h"
#include "tblist.h"

// minime listen time (in second) for a tune to become counted
#define MIN_LISTEN_TIME 10

// default size for both number and second
#define DEFAULT_SIZE 100

#ifdef HAVE_UNIX
#  define TSID2DIR ".tsid2"
#else
#  define TSID2DIR "tsid2"
#endif

// show debug string if DEBUG is defined
#ifdef DEBUG_TSID2
  #define WDEBUG2(a) \
    cerr << "[TSID2] " << a;
#else
  #define WDEBUG2(a) ;
#endif

/**
 * TSID2 class
 *
 * For speed up the file management in a sidplayer, a writing and reading of a
 * file is made only when the user pass to another tune, or close the program.
 * A minor overload is for testing the esistence of the directory tree of TSID.
 *
 * @author ice
 * @version 1.00 25/02/2002
 */
class TSID2 {
  public:

    // a TSID2 record of information
    struct tsidRecord {
      char *MD5;                   // MD5 fingerprinter
      char *absSidFileName;        // last known absolute path and name of the file
      unsigned int time;           // listen time
      unsigned int tune;           // subtune number
      unsigned int length;         // length of the song (0 if not known)
    };

    /**
     * Construct the TSID2 with passed base directory
     *
     * @param engine the type of the engine
     * @param number_t the number of time item to store in the list
     * @param number_n the number of number item  to store in the list
     * @param absBasePath the base path where there are the sidt2 files
     */
    TSID2(TBLIST::engineType engine=TBLIST::DINAMIC, unsigned int number_t=DEFAULT_SIZE,
          unsigned int number_n=DEFAULT_SIZE, const char *absBasePath=NULL);

    /**
     * Destruct the TSID
     */
    ~TSID2();

    /**
     * Return the state of TSID2: false means that TSID2 cannot continue
     */
    operator bool() {
      return tsidStatus;
    }

    /**
     * Get the version of TSID2 as a formatted string
     *
     * @return the version of TSID2
     */
    inline char *getVersion() {
      return versionString;
    }

    /**
     * Get Version of TSID2 Package
     */
    static inline const char *getTSID2Version() {
      return TSID2_VERSION;
    }

    /**
     * Add the listen time to the corrisponding MD5 file
     * Use MD5 as tune name, absFilePath as absolute path of the last know
     * position of the tune; length<>0 if it is known.
     *
     * @param absSidFileName last known position of absolute file path and name
     * @param time the number of listened seconds
     * @param tune the subtune number
     * @param length the length of the tune or 0
     * @return true if the operation is ok
     */
    bool addTime (const char *MD5, const char *absSidFileName, unsigned int time,
                  unsigned int tune, unsigned int length=0);

    /**
     * Add the listen time using a struct record of information
     *
     * @param tsidRecord the record of information
     * @return true if the operation is ok
     */
    bool addTime (struct tsidRecord rec);

    /**
     * Flush the information not saved to file
     *
     * @return true if operation is ok
     */
    bool flush();

    /**
     * Get the tblist pointer.
     * This is provided for getting the hotlists
     *
     * @return the tblist pointer
     */
    inline TBLIST* getTblist() {
      return pList;
    }

  private:
    // true if tsid2 can continue to work correctly
    bool tsidStatus;

    // sidt2 file manager pointer
    SIDT2 *pSidt;

    // hot list pointer
    TBLIST *pList;

    // Version number/copyright string
    char *versionString;

    // Home directory of user
    char *homeDir;

    // the base path where there are the sidt2 files
    char *absBasePath;

    // the MD5 of the buffered tune 
    char *bufferMD5;

    /**
     * Get home directory of user
     * The private variables homeDir is changed
     *
     * @return true if operation is ok
     */
    bool getHomeDir();

    /**
     * Convert platform specific directory seperator to ansi
     *
     * @param filename the filename to made with '/'
     */
    void convertDirSeperator(char *filename);
};

#endif // _TSID2_H
